/* =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=- */
/*          EXIFPROBE - TIFF/JPEG/EXIF image file probe               */
/* =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=- */
/* Copyright (C) 2002 by Duane H. Hesser. All rights reserved.        */
/*                                                                    */
/* See the file LICENSE.EXIFPROBE for terms of use.                   */
/* =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=- */

#ifndef lint
static char *ModuleId = "@(#) $Id: maker_asahi.c,v 1.2 2003/04/26 14:15:16 alex Exp $";
#endif

/* =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=- */
/* Asahi/Pentax camera maker-specific routines                        */
/* =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=- */
/* This information is from ab image by Johannes Tschebisch at:       */
/* http://www.jojotsch.de/downloads/jojothumb/beispiele/              */
/*                             html_exif/bilder/bilder.html           */
/*                                                                    */
/* A bare minimum is known at this time (4/22/03)...just the PrintIM  */
/* and JPEG Thumbnail tags.                                           */
/* =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=- */

#include <stdio.h>
#include <string.h>
#include "defs.h"
#include "extern.h"
#include "misc.h"
#include "maker.h"

/* Find the identifying number assigned to known Asahi/Pentax camera  */
/* models. This number is used to dispatch print and interpret        */
/* routines approopriate to the current image.                        */

int
asahi_model_number(char *model,char *software)
{
    int number = GENERIC;

    if(model)
    {
        switch(*model)
        {
                case 'P':
                    if(strncmp(model,"PENTAX Optio430RS",17) == 0)
                        number = ASAHI_PENTAX430RS;
                    else if(strncmp(model,"PENTAX Optio 330",16) == 0)
                        number = ASAHI_PENTAX330;
                    else if(strncmp(model,"PENTAX Optio 430",16) == 0)
                        number = ASAHI_PENTAX430;
                break;
            default:
                break;
        }
    }
    return(number);
}


/* Dispatch a print routine for direct values in Asahi/Pentax         */
/* cameras, based upon model                                          */

void
print_asahi_makervalue(struct ifd_entry *entry_ptr,int make,int model,
                    char *prefix)
{

    if(entry_ptr && (PRINT_VALUE))
    {
        switch(model)
        {
            case GENERIC:
            default:
                print_asahi1_makervalue(entry_ptr,make,model,prefix);
                asahi1_interpret_value(entry_ptr);
                break;
        }
    }
}

/* Model-specific print routine for Asahi/Pentax cameras. This        */
/* routine is responsible for picking off any direct entry tags which */
/* are peculiar and will not be handled properly by print_value()     */
/* (usually UNDEFINED values which fit in the 4-byte entry value). If */
/* there are no such entries, this function simply calls              */
/* print_value().                                                     */

void
print_asahi1_makervalue(struct ifd_entry *entry_ptr,int make,int model,
                    char *prefix)
{
    if(entry_ptr && (PRINT_VALUE))
    {
        switch(entry_ptr->tag) 
        {
            default:
                print_value(entry_ptr,PREFIX);
                break;
        }
    }
}


/* Dispatch a routine to decode and print offset values for Asahi/Pentax     */
/* cameras.                                                           */

void
print_asahi_offset_makervalue(FILE *inptr,unsigned short byteorder,
    struct ifd_entry *entry_ptr,unsigned long offset_ref,char *prefix,
    int indent,int addrwidth,int make,int model,int at_offset)
{

    if(entry_ptr && (PRINT_VALUE))
    {
        switch(model)
        {
            /* currently only one routine available                       */
            default:
                asahi1_offset_makervalue(inptr,byteorder,entry_ptr,
                                            offset_ref,prefix,indent,
                                            addrwidth,make,model,at_offset);
                break;
        }
    }
}

/* Model-specific routine to print UNDEFINED values found at offsets  */
/* in Asahi/Pentax makernotes.                                        */

void
asahi1_offset_makervalue(FILE *inptr,unsigned short byteorder,
    struct ifd_entry *entry_ptr,unsigned long offset_ref,char *prefix,
    int indent,int addrwidth,int make,int model,int at_offset)
{
    unsigned long value_offset;
    unsigned long dumplength;
    unsigned short marker;
    char *nameoftag;

    if(entry_ptr)
    {
        nameoftag = maker_tagname(entry_ptr->tag,make,model); 
        value_offset = offset_ref + entry_ptr->value;

        switch(entry_ptr->tag)
        {
            case 0x2000: /* JPEG Thumbnail in TIFF MakerNote          */
                if(at_offset)
                {
                    print_tag_address(VALUE_AT_OFFSET,value_offset,indent,prefix,addrwidth);
                    extraindent(SUBINDENT);
                    printf("Start of JPEG Thumbnail");
                }
                if(!(PRINT_OFFSET))
                {
                    if(PRINT_BOTH_VALUE)
                        printf("@%#lx=%lu",value_offset,value_offset);
                    else if(PRINT_HEX_VALUE)
                        printf("@%#lx",value_offset);
                    else if(PRINT_DEC_VALUE)
                        printf("@%lu",value_offset);
                }
                printf(" length %ld",entry_ptr->count);
                marker = read_imagemarker(inptr,value_offset);
                if(PRINT_SEGMENT)
                {
                    putchar('\n');
                    (void)process_jpeg_segments(inptr,value_offset,
                                                marker,entry_ptr->count,
                                                indent+(SUBINDENT * 2),addrwidth);
                    print_tag_address(SEGMENT,value_offset + entry_ptr->count - 1,
                                                indent,"-",addrwidth);
                    extraindent(SUBINDENT);
                    printf("End of JPEG Thumbnail");
                }
                /* make certain we're at the end unless 'inptr' bad   */
                fseek(inptr,value_offset + entry_ptr->count,0);
                break;
            case 0x0e00: /* PrintIM (Epson Print Image matching)      */
                if(at_offset)
                {
                    print_tag_address(ENTRY,value_offset,indent,prefix,addrwidth);
                    print_makertagid(entry_ptr,19," : ",SUBINDENT,make,model);
                }
                else if(PRINT_OFFSET)
                        printf(" = ");
                printf("UNDEFINED, count %lu", entry_ptr->count);
                if(Max_undefined == 0)
                    printred(" (not dumped, use -U)");
                else
                {
                    if((Max_undefined == DUMPALL)
                        || (Max_undefined > entry_ptr->count))
                            dumplength = entry_ptr->count;
                    else 
                        dumplength = Max_undefined;
                    putchar('\n');
                    hexdump(inptr,value_offset,entry_ptr->count,dumplength,12,
                                indent,SUBINDENT,addrwidth);
                }
                /* make certain we're at the end unless 'inptr' bad   */
                fseek(inptr,value_offset + entry_ptr->count,0);
                break;
            default:
                print_generic_offset_makervalue(inptr,byteorder,entry_ptr,
                                        offset_ref,prefix,indent,
                                        addrwidth,make,model,at_offset);
                break;
        }
    }
}


/* Asahi/Pentax-specific tagnames for makernotes.                     */

char *
maker_asahi_tagname(unsigned short tag,int model)
{
    char *tagname = (char *)0;

    /* Check specific models first                                    */
    switch(model)
    {
        /* Add special-case models here                               */
        default:
            break;
    }

    /* If no model-specific tag is found, use a "generic"             */
    /* Asahi/Pentax tag                                               */
    if(tagname == NULL)
    {
        switch(tag)
        {
            case 0x0e00: tagname = "PrintIMData"; break;
            case 0x2000: tagname = "JPEG_Thumbnail"; break;
            default: break;
        }
    }
    return(tagname);
}


/* Report the "meaning" of tag values found at offsets in a           */
/* Asahi/Pentax MakerNote IFD entry (not at an offset).               */

void
asahi1_interpret_value(struct ifd_entry *entry_ptr)
{
/* nothing known yet                                                  */
}
