/* =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=- */
/*          EXIFPROBE - TIFF/JPEG/EXIF image file probe               */
/* =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=- */
/* Copyright (C) 2002 by Duane H. Hesser. All rights reserved.        */
/*                                                                    */
/* See the file LICENSE.EXIFPROBE for terms of use.                   */
/* =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=- */

#ifndef lint
static char *ModuleId = "@(#) $Id: print_maker.c,v 1.16 2003/08/02 01:03:52 alex Exp $";
#endif

#include <stdio.h>
#include "defs.h"
#include "extern.h"
#include "maker.h"
#include "misc.h"
#include "tags.h"


/* Print an IFD entry presumed to be from a MakerNote.                */

/* This differs from print_entry() in that it dispatches              */
/* maker_tagname() to get tag names specific to a specific make and   */
/* (possibly) model, and calls print_makervalue() to intercept any    */
/* tags which may require special treatment due to maker oddities.    */
/* The maker-specific routines dispatched by print_makervalue() will  */
/* also handle interpretation of the tag values if known.             */

unsigned long
print_makerentry(FILE *inptr,unsigned short byteorder,int addrwidth,
                    struct ifd_entry *entry_ptr,unsigned long fileoffset_base,
                    int indent,int make,int model)
{
    int value_is_offset = 0;
    unsigned long endoffset = 0UL;

    value_is_offset = is_offset(entry_ptr);
    if(PRINT_ENTRY)
    {

        print_makertaginfo(entry_ptr,SUBINDENT,make,model);
        if((entry_ptr->value_type == 0) || (entry_ptr->value_type > DOUBLE))
        {
            printf("INVALID (%lu)",entry_ptr->value);
            endoffset = ftell(inptr);
        }
        else if(entry_ptr->count == 0)
        {
            if(PRINT_VALUE)
                printf("EMPTY (%lu)",entry_ptr->value);
            endoffset = ftell(inptr);
        }
        else if(value_is_offset)
        {
            if(PRINT_BOTH_OFFSET)
                printf("@%#lx=%lu",entry_ptr->value + fileoffset_base,
                        entry_ptr->value + fileoffset_base);
            else if(PRINT_HEX_OFFSET)
                printf("@%#lx",entry_ptr->value + fileoffset_base);
            else if(PRINT_DEC_OFFSET)
                printf("@%lu",entry_ptr->value + fileoffset_base);

            if(fileoffset_base && (PRINT_ENTRY_RELOFFSET))
                printf(" (%lu)",entry_ptr->value);

            if((PRINT_VALUE) && !(PRINT_VALUE_AT_OFFSET))
            {
                endoffset = print_offset_makervalue(inptr,byteorder,
                                            entry_ptr,fileoffset_base,
                                            "@",indent + SUBINDENT,
                                            addrwidth,make,model,0);
                return(endoffset);
            }
            else
                endoffset = fileoffset_base + entry_ptr->value + 
                    (value_type_size(entry_ptr->value_type) * entry_ptr->count);
        }
        else if(PRINT_VALUE)
            print_makervalue(entry_ptr,make,model,PREFIX);
        (void)putchar('\n');
    }
    return(endoffset);
}

/* Repeat the tag identifier for values printed at an offset. This is */
/* called only from the "*makervalue-at_offset()" routines when       */
/* printing offset values at the offset (rather than "inline"), and   */
/* repeats only one of the identifiers (preferably tagname) followed  */
/* by the value separator (if values are printed at all). If no tag   */
/* identifier is enabled in Print_options, the tag name will be       */
/* printed, as in the entry line printed by print_makertaginfo().     */

/* The caller will normally pass the 'sep' argument as ' = ' if an    */
/* actual value is to follow, or ': ' if a description (e.g. length   */
/* of UNDEFINED segment which is not expanded).                       */

void
print_makertagid(struct ifd_entry *entry_ptr,int width, char *sep,
                    int indent,int make,int model)
{
    char *nameoftag;
    char *nameof_value_type;

    if(entry_ptr)
    {
        extraindent(indent);
        if((PRINT_TAGNAME) || !(PRINT_TAGNO))
        {
            nameoftag = maker_tagname(entry_ptr->tag,make,model);
            printf("%-*.*s",width,width,nameoftag);
        }
        else if(PRINT_HEX_TAGNO)
            printf("<%#06x>",entry_ptr->tag & 0xffff);
        else
            printf("<%5u>",entry_ptr->tag & 0xffff);
        if(PRINT_VALUE)
            printf("%s",sep ? sep : " = ");
    }
}

/* Print the part of a MakerNote entry describing the entry tag,      */
/* including it's tag number, name and type. Only the items enabled   */
/* in "Print_options" are printed, except that if the value is to be  */
/* printed but neither tagname or tag number is enabled, print the    */
/* tag name anyway (it doesn't make much sense to print the value     */
/* without some sort of identifer).                                   */

/* In LIST mode (when not printing SECTION info) prefix the tag name  */
/* with a section identifier, so that the resulting names may be      */
/* recognized by patterns or regular expressions applied to the       */
/* output (e.g. 'egrep') in post-processing.                          */

void
print_makertaginfo(struct ifd_entry *entry_ptr,int indent,int make,int model)
{
    char *nameoftag;
    char *nameof_value_type;

    if(entry_ptr)
    {
        extraindent(indent);
        if(PRINT_BOTH_TAGNO)
            printf("<%#06X=%5u> ",entry_ptr->tag & 0xffff,entry_ptr->tag & 0xffff);
        else if(PRINT_HEX_TAGNO)
            printf("<%#06X> ",entry_ptr->tag & 0xffff);
        else  if(PRINT_DEC_TAGNO)
            printf("<%5u> ",entry_ptr->tag & 0xffff);
        if((PRINT_TAGNAME) || ((PRINT_VALUE) && !(PRINT_TAGNO)))
        {
            nameoftag = maker_tagname(entry_ptr->tag,make,model);
            if((PRINT_LIST))
                printf("Exif_Maker_");
            printf("%-*.*s",TAGWIDTH - 11 + PREFIXLEN,
                    TAGWIDTH - 11 + PREFIXLEN,nameoftag);
        }
        if(PRINT_TYPE)
        {
            nameof_value_type = value_type_name(entry_ptr->value_type);
            printf(" [%2u=%-9.9s %5lu] ",entry_ptr->value_type,
                    nameof_value_type,entry_ptr->count);
        }
        if(PRINT_VALUE)
            printf(" = "); 
    }
}

/* Dispatch maker-specific routines to print (and possibly interpret) */
/* non-offset values in MakerNote IFDs. If the make is not known, the */
/* normal print_value() routine will be called, and the               */
/* maker-specific routines are free to do the same if the tag doesn't */
/* need special handling (such as a 4 byte UNDEFINED which should be  */
/* printed as ascii). The maker-specific routines can also call       */
/* routines to interpret the value (e.g "On", "Off", etc.) according  */
/* to the requirements of the make and model.                         */

void
print_makervalue(struct ifd_entry *entry_ptr,int make,int model,char *prefix)
{
    char *modelname,*swname;

    if(entry_ptr)
    {
        /* Override real names if -m, -l, -s                          */
        if(Use_Make_name)
            make = maker_number(Use_Make_name);
        modelname = Use_Model_name ? Use_Model_name : Model_name;
        swname = Use_Software_name ? Use_Software_name : Software_name;
        model = model_number(make,modelname,swname);

        switch(make)
        {
            case MAKER_AGFA:
                print_agfa_makervalue(entry_ptr,make,model,prefix);
                break;
            case MAKER_ASAHI:   /* Pentax                             */
                print_asahi_makervalue(entry_ptr,make,model,prefix);
                break;
            case MAKER_CANON:
                print_canon_makervalue(entry_ptr,make,model,prefix);
                break;
            case MAKER_CASIO:
                print_casio_makervalue(entry_ptr,make,model,prefix);
                break;
            case MAKER_EPSON:
                print_epson_makervalue(entry_ptr,make,model,prefix);
                break;
            case MAKER_FUJIFILM:
                print_fujifilm_makervalue(entry_ptr,make,model,prefix);
                break;
            case MAKER_HP:
                print_value(entry_ptr,prefix);
                break;
            case MAKER_KODAK:
                print_value(entry_ptr,prefix);
                break;
            case MAKER_KONICA:
                print_value(entry_ptr,prefix);
                break;
            case MAKER_KYOCERA:
                print_value(entry_ptr,prefix);
                break;
            case MAKER_LEICA:
                print_leica_makervalue(entry_ptr,make,model,prefix);
                break;
            case MAKER_MINOLTA:
                print_minolta_makervalue(entry_ptr,make,model,prefix);
                break;
            case MAKER_NIKON:
                print_nikon_makervalue(entry_ptr,make,model,prefix);
                break;
            case MAKER_OLYMPUS:
                print_olympus_makervalue(entry_ptr,make,model,prefix);
                break;
            case MAKER_PANASONIC:
                print_value(entry_ptr,prefix);
                break;
            case MAKER_RICOH:
                break;
            case MAKER_SANYO:
                print_sanyo_makervalue(entry_ptr,make,model,prefix);
                break;
            case MAKER_SONY:
                print_value(entry_ptr,prefix);
                break;
            case MAKER_TOSHIBA:
                print_value(entry_ptr,prefix);
                break;
            case MAKER_TRAVELER:
                print_value(entry_ptr,prefix);
                break;
            default:                /* No print routine available     */
                print_value(entry_ptr,prefix);
                break;
        }
    }
}

/* Print values found at offsets in a MakerNote entry by dispatching  */
/* maker-specific routines which know how to interpret UNDEFINED      */
/* values for that maker. These routines may also intercept "defined" */
/* values which are handled strangely, such as a "long" value which   */
/* is really an offset, or a 4-byte UNDEFINED value which is really   */
/* an offset to an UNDEFINED blob. If a maker-specific routine is     */
/* called, it must handle all values for that maker (possibly be      */
/* calling the generic routine). The generic and maker-specific       */
/* routines are all 'void' functions, which should assure that the    */
/* file pointer is moved past the subject data. This routine checks   */
/* for errors on the file pointer and returns the current file        */
/* offset, or 0 if a read error occurred. In the latter case, the     */
/* error is cleared and the caller may attempt to recover for its     */
/* next assignment.                                                   */

unsigned long
print_offset_makervalue(FILE *inptr,unsigned short byteorder,
    struct ifd_entry *entry_ptr,unsigned long fileoffset_base,char *prefix,
    int indent,int addrwidth,int make,int model,int at_offset)
{
    unsigned long endoffset = 0L;
    char *modelname,*swname;

    if(inptr && entry_ptr)
    {
        endoffset = fileoffset_base + entry_ptr->value + 
            (value_type_size(entry_ptr->value_type) * entry_ptr->count);

        /* Override real names if -m, -l, -s                          */
        if(Use_Make_name)
            make = maker_number(Use_Make_name);
        modelname = Use_Model_name ? Use_Model_name : Model_name;
        swname = Use_Software_name ? Use_Software_name : Software_name;
        model = model_number(make,modelname,swname);

        switch(make)
        {
            case MAKER_AGFA:
                print_agfa_offset_makervalue(inptr,byteorder,entry_ptr,
                                            fileoffset_base,prefix,indent,
                                            addrwidth,make,model,at_offset);
                break;
            case MAKER_ASAHI:   /* Pentax                         */
                print_asahi_offset_makervalue(inptr,byteorder,entry_ptr,
                                            fileoffset_base,prefix,indent,
                                            addrwidth,make,model,at_offset);
                break;
            case MAKER_CANON:
                print_canon_offset_makervalue(inptr,byteorder,entry_ptr,
                                            fileoffset_base,prefix,indent,
                                            addrwidth,make,model,at_offset);
                break;
            case MAKER_CASIO:
                print_casio_offset_makervalue(inptr,byteorder,entry_ptr,
                                            fileoffset_base,prefix,indent,
                                            addrwidth,make,model,at_offset);
                break;
            case MAKER_EPSON:
                print_epson_offset_makervalue(inptr,byteorder,entry_ptr,
                                            fileoffset_base,prefix,indent,
                                            addrwidth,make,model,at_offset);
                break;
            case MAKER_FUJIFILM:
                print_fujifilm_offset_makervalue(inptr,byteorder,entry_ptr,
                                            fileoffset_base,prefix,indent,
                                            addrwidth,make,model,at_offset);
                break;
            case MAKER_HP:
                print_generic_offset_makervalue(inptr,byteorder,entry_ptr,
                                            fileoffset_base,prefix,indent,
                                            addrwidth,make,model,at_offset);
                break;
            case MAKER_KODAK:
                print_generic_offset_makervalue(inptr,byteorder,entry_ptr,
                                            fileoffset_base,prefix,indent,
                                            addrwidth,make,model,at_offset);
                break;
            case MAKER_KYOCERA:
                print_generic_offset_makervalue(inptr,byteorder,entry_ptr,
                                            fileoffset_base,prefix,indent,
                                            addrwidth,make,model,at_offset);
                break;
            case MAKER_KONICA:
                print_generic_offset_makervalue(inptr,byteorder,entry_ptr,
                                            fileoffset_base,prefix,indent,
                                            addrwidth,make,model,at_offset);
                break;
            case MAKER_LEICA:
                print_leica_offset_makervalue(inptr,byteorder,entry_ptr,
                                            fileoffset_base,prefix,indent,
                                            addrwidth,make,model,at_offset);
                break;
            case MAKER_MINOLTA:
                print_minolta_offset_makervalue(inptr,byteorder,entry_ptr,
                                            fileoffset_base,prefix,indent,
                                            addrwidth,make,model,at_offset);
                break;
            case MAKER_NIKON:
                print_nikon_offset_makervalue(inptr,byteorder,entry_ptr,
                                            fileoffset_base,prefix,indent,
                                            addrwidth,make,model,at_offset);
                break;
            case MAKER_OLYMPUS:
                print_olympus_offset_makervalue(inptr,byteorder,entry_ptr,
                                            fileoffset_base,prefix,indent,
                                            addrwidth,make,model,at_offset);
                break;
            case MAKER_PANASONIC:
                print_generic_offset_makervalue(inptr,byteorder,entry_ptr,
                                            fileoffset_base,prefix,indent,
                                            addrwidth,make,model,at_offset);
                break;
            case MAKER_RICOH: /* ###%%% generic? */
                break;
            case MAKER_SANYO:
                print_sanyo_offset_makervalue(inptr,byteorder,entry_ptr,
                                            fileoffset_base,prefix,indent,
                                            addrwidth,make,model,at_offset);
                break;
            case MAKER_SONY:
                print_generic_offset_makervalue(inptr,byteorder,entry_ptr,
                                            fileoffset_base,prefix,indent,
                                            addrwidth,make,model,at_offset);
                break;
            case MAKER_TOSHIBA:
                print_generic_offset_makervalue(inptr,byteorder,entry_ptr,
                                            fileoffset_base,prefix,indent,
                                            addrwidth,make,model,at_offset);
                break;
            case MAKER_TRAVELER:
                print_generic_offset_makervalue(inptr,byteorder,entry_ptr,
                                            fileoffset_base,prefix,indent,
                                            addrwidth,make,model,at_offset);
                break;
            default:
                print_generic_offset_makervalue(inptr,byteorder,entry_ptr,
                                            fileoffset_base,prefix,indent,
                                            addrwidth,make,model,at_offset);
                break;
        }
        if(PRINT_VALUE) /* ###%%% */
            putchar('\n');
    }
    else
    {
        printred(" invalid call to print_offset_makervalue ");
        if(inptr == (FILE *)0)
            printred("no input file pointer ");
        if(entry_ptr == (struct ifd_entry *)0)
            printred(" null entry pointer");
    }
    return(endoffset);
}

/* Print the value found at an offset in a generic MakerNote, if no   */
/* maker-specific routine wants to print it. Similar to               */
/* print_offset_value(). The routine is expected to read or skip all  */
/* data associated with the entry. The caller is expected to check    */
/* for read errors.                                                   */

void
print_generic_offset_makervalue(FILE *inptr,unsigned short byteorder,
    struct ifd_entry *entry_ptr,unsigned long fileoffset_base,char *prefix,
    int indent,int addrwidth,int make,int model,int at_offset)
{
    unsigned long value_offset;
    unsigned long dumplength;
    char *nameoftag;

    if((value_type_size(entry_ptr->value_type) * entry_ptr->count) > 4)
    {
        value_offset = fileoffset_base + entry_ptr->value;
        if(PRINT_VALUE)
        {
            nameoftag = maker_tagname(entry_ptr->tag,make,model); 
            if(at_offset)
            {
                print_tag_address(ENTRY,value_offset,indent,prefix,addrwidth);
                print_makertagid(entry_ptr,19," = ",SUBINDENT,make,model);
            }
            else if(PRINT_OFFSET)
                    printf(" = ");
            switch(entry_ptr->value_type)
            {
                case UNDEFINED: 
                    printf("UNDEFINED, count %lu", entry_ptr->count);
                    if(Max_undefined == 0)
                        printred(" (not dumped, use -U)");
                    else
                    {
                        if((Max_undefined == DUMPALL)
                            || (Max_undefined > entry_ptr->count))
                                dumplength = entry_ptr->count;
                        else 
                            dumplength = Max_undefined;
                        putchar('\n');
                        hexdump(inptr,entry_ptr->value + fileoffset_base,
                                    entry_ptr->count,dumplength,12,
                                    indent,SUBINDENT,addrwidth);
                    }
                    /* make certain we're at the end                  */
                    fseek(inptr,entry_ptr->value + fileoffset_base + entry_ptr->count,0);
                    break;
                case ASCII:
                    print_ascii(inptr,entry_ptr->count,value_offset); 
                    break;
                case BYTE:
                    print_ubytes(inptr,entry_ptr->count,value_offset);
                    break;
                case SBYTE:
                    print_sbytes(inptr,entry_ptr->count,value_offset);
                    break;
                case SHORT:
                    print_ushort(inptr,entry_ptr->count,byteorder,value_offset);
                    break;
                case SSHORT:
                    print_sshort(inptr,entry_ptr->count,byteorder,value_offset);
                    break;
                case LONG:
                    print_ulong(inptr,entry_ptr->count,byteorder,value_offset);
                    break;
                case SLONG:
                    print_slong(inptr,entry_ptr->count,byteorder,value_offset);
                    break;
                case RATIONAL:
                    print_urational(inptr,entry_ptr->count,byteorder,value_offset);
                    break;
                case SRATIONAL:
                    print_srational(inptr,entry_ptr->count,byteorder,value_offset);
                    break;
                case FLOAT:
                    print_float(inptr,entry_ptr->count,byteorder,value_offset);
                    break;
                case DOUBLE:
                    print_double(inptr,entry_ptr->count,byteorder,value_offset);
                    break;
                default:
                    printf(" INVALID TYPE %#x",entry_ptr->value_type);
                    break;
            }
        }
    }
}
